/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIContainerBoxObject.idl"

/**
 * @deprecated Please consider using nsIContainerBoxObject.
 */

[scriptable, uuid(e3800a23-5b83-49aa-b18c-efa1ac5416e0)]
interface nsIEditorBoxObject : nsIContainerBoxObject
{
};

